;-- PEEK C64 Memory

;===============================================================
.DATA

 EXTRN PC:WORD

              ; 0/1xxx 2/3xxx 4/5xxx 6/7xxx
PeekPCTable  dw pcRAM, pcRAM, pcRAM, pcRAM
              ; 8/9xxx A/Bxxx C/Dxxx E/Fxxx
 	     dw pcRAM, pcROM, pcRAM, pcROM

	      ; 0xxx 1xxx,2xxx,3xxx,4xxx,5xxx,6xxx,7xxx
PeekTable    dw pRAM,pRAM,pRAM,pRAM,pRAM,pRAM,pRAM,pRAM
	      ; 8xxx,9xxx,Axxx,Bxxx,Cxxx,Dxxx,Exxx,Fxxx
 	     dw pRAM,pRAM,pROM,pROM,pRAM,pCIO,pROM,pROM

	      ; D0xx D1xx D2xx D3xx D4xx D5xx D6xx D7xx
PeekCIOTable dw rVIC,rVIC,rVIC,rVIC,rSID,rSID,rSID,rSID
	      ; D8xx D9xx DAxx DBxx DCxx DDxx DExx DFxx
	     dw rCOL,rCOL,rCOL,rCOL,rCI1,rCI2,rCOL,rEMU

	      ; 0xxx 1xxx,2xxx,3xxx,4xxx,5xxx,6xxx,7xxx
DeekTable    dw dRAM,dRAM,dRAM,dRAM,dRAM,dRAM,dRAM,dRAM
	      ; 8xxx,9xxx,Axxx,Bxxx,Cxxx,Dxxx,Exxx,Fxxx
 	     dw dRAM,dRAM,dROM,dROM,dRAM,dCIO,dROM,dROM

;===============================================================
.CODE

;--- BX = PEEK(PC) AND 255

PIP MACRO Seg
 IFDEF TRAP4
  MOV DX,Seg:[DI+1]
 ENDIF
ENDM

BEGINP PeekPC
 XOR EBX,EBX
 XOR EDX,EDX

 ;------------
; TRAP4 equ YES

 IFDEF TRAP4
  MOV BX,PC
  INC PC
  Call PeekBX
  MOV  BX,DX
  XOR  DH,DH
  MOV  DL,AL
  JMP32 EDX, OpTable
 ENDIF

 ;------------

 MOV SI,PC
 MOV BX,SI
 SHR BX,13
 JMP32 EBX, PeekPCTable ;-- JMP [2*EBX+PeekPCTable]

comment ! TRAP4

 MOV BX,PC
 Call PeekBX ; AL
 MOV DL,AL   ; OpCode

 MOV BX,PC   ; Fetch
 INC BX
 MOV PC,BX

 Call PeekBX ; AL
 MOV CL,AL   ; Imm

 MOV BX,PC
 INC BX
 Call PeekBX
 MOV CH,AL
 MOV BX,CX   ; Abso

 JMP32 EDX, OpTable
TRAP4 !

PeekPC ENDP


BEGINP pcRAM
 IF DEBUG
  CALLP dbgPCRAM
 ENDIF
 MOV DL,RAM:[SI]
 INC SI
 MOV BX,RAM:[SI]
 MOV PC,SI
 JMP32 EDX, OpTable	;-- JMP [2*EDX+Offset OpTable]
pcRAM ENDP

BEGINP pcROM
 IF DEBUG
  CALLP dbgPCROM
 ENDIF
 MOV DL,ROM:[SI]
 INC SI
 MOV BX,ROM:[SI]
 MOV PC,SI
 JMP32 EDX, OpTable	;-- JMP [2*EDX+Offset OpTable]
pcROM ENDP


;--- AL=PEEK (DI) AND 255
BEGINP PeekBX
 MOV DI,BX
 SHR BX,12
 XOR AH,AH
 JMP32 EBX, PeekTable	;-- JMP [2*EBX+Offset PeekTable]
PeekBX ENDP

BEGINP pRAM
 PIP RAM
 MOV AL,RAM:[DI]
 RET
pRAM ENDP

BEGINP pROM
 PIP ROM
 MOV AL,ROM:[DI]
 RET
pROM ENDP

;--- PEEK($Dxxx)
BEGINP pCIO
 PIP CIO
 MOV BX,DI
 SHR BX,8
 AND BX,0Fh
 JMP32 EBX, PeekCIOTable ;-- JMP [2*EBX+Offset PeekCIOTable]
pCIO ENDP

BEGINP rVIC
 AND DI,0F03Fh ; Patch VIC Address !
 MOV BX,DI
 AND BX,3Fh
 JMP32 EBX, PeekVICTable
rVIC ENDP

BEGINP rSID
IFE DEBUG
 MOV AL,SIDPoke ; read last poked value
ENDIF
 RET
rSID ENDP

BEGINP rCOL
 MOV AL,CIO:[DI]
 RET
rCOL ENDP

BEGINP rCI1
 AND DI,0FF0Fh ; Patch CIA Address !
 MOV BX,DI
 AND BX,15
 JMP32 EBX, PeekCIA1     ;-- JMP [2*EBX+Offset PeekCIA1]
rCI1 ENDP

BEGINP rCI2
 AND DI,0FF0Fh ; Patch CIA Adresse !
 MOV BX,DI
 AND BX,15
 JMP32 EBX, PeekCIA2     ;-- JMP [2*EBX+Offset PeekCIA2]
rCI2 ENDP

BEGINP rEMU
 CMP DI,0DFA0h
 JB  rCOL
 MOV AL,DS:[DI+CBM64NAME-0DFA0h]
 CMP DI,0DFFFh
 JNE rEMU_Done
  NOT AL
  MOV AA55,AL ; AAh<->55h
 rEMU_Done:
 RET
rEMU ENDP

;--- BX = DEEK (BX)
BEGINP DeekBX
 XOR EDX,EDX
 MOV DX,BX
 SHR DX,12
 JMP32 EDX,DeekTable
DeekBX ENDP

BEGINP dRAM
 IF DEBUG
  CALLP dbgDRAM
 ENDIF

 MOV AL,RAM:[BX]
 INC BL
 MOV AH,RAM:[BX]
 MOV BX,AX

 RET
dRAM ENDP

BEGINP dROM
 IF DEBUG
  CALLP dbgDROM
 ENDIF

 MOV AL,ROM:[BX]
 INC BL
 MOV AH,ROM:[BX]
 MOV BX,AX

 RET
dROM ENDP

;--- DEEK($Dxxx)
BEGINP dCIO
 IF DEBUG
  CALLP dbgDCIO
 ENDIF

 MOV BX,DI
 MOV AL,CIO:[BX]
 INC BL
 MOV AH,CIO:[BX]
 MOV BX,AX

 RET
dCIO ENDP

