Unit VGA;

Interface

{$G+}

Procedure TweakMode;
Procedure TextMode;
Procedure SelectPage(Page:word);
Procedure PutPixel(x,y,c:word);
Procedure Box(Left,Top,Width,Height,Color:word);
Procedure Frame3D(left,top,width,height,Light,Shadow,Background:word);
Procedure OutTextXY(x,y,c:word;const Text:String);
Procedure HLine(x,y,w,c:word);

Const
 clBlack      =0;
 clWhite      =1;
 clRed        =2;
 clCyan       =3;
 clPurple     =4;
 clGreen      =5;
 clBlue       =6;
 clYellow     =7;
 clOrange     =8;
 clBrown      =9;
 clLightRed   =10;
 clDarkGrey   =11;
 clGrey       =12;
 clLightGreen =13;
 clLightBlue  =14;
 clLightGrey  =15;

Const
 VideoSeg:word=$A000;
 LineChars=90; { Pixels = 4*128 = 512 }


Implementation

uses
 c64rom,data;

Const
 Palette:array[0..15,0..2] of byte=(
  ($00,$00,$00), { 0 - Black    }
  ($3F,$3F,$3F), { 1 - White    }
  ($2E,$00,$05), { 2 - Red      }
  ($00,$30,$2B), { 3 - Cyan     }
  ($25,$00,$26), { 4 - Purple   }
  ($09,$33,$00), { 5 - Green    }
  ($01,$04,$2C), { 6 - Blue     }
  ($3F,$3F,$00), { 7 - Yellow   }
  ($26,$15,$00), { 8 - Orange   }
  ($14,$08,$05), { 9 - Brown    }
  ($3F,$14,$1E), { 10- Lt.Red   }
  ($12,$12,$12), { 11- Dk.Grey  }
  ($1C,$1C,$1C), { 12- Grey     }
  ($17,$3F,$15), { 13- Lt.Green }
  ($15,$1C,$3A), { 14- Lt.Blue  }
  ($2B,$2B,$2B)  { 15- Lt.Grey  }
 );


 TweakParm:array[1..17] of word = (
  $6B00,    { Horizontal total          }
            { 107-5 -> 102 * 8 = 816 pixels per line }
  $5901,    { Horizontal displayed      }
            {  89   ->  89 * 8 = 712 visible pixels }
  $5A02,    { Start horizontal blanking }
            {  90   ->  90 * 8 = Right border at pixel 720 }
  $8003,    { 8E - end horizontal blanking   }

  $5E04,    { Start H sync.             }
            {  94   ->  94 * 8 = End of Line at pixel 752 }
  $0A05,    { 8A - end H sync.               }
            {  10   ->  10 * 8 = Restart Line at pixel 80 }
  $0D06,    { Vertical total            }
            { + Overflow (5,0) = 512    }
            { 512+13= 525 lines         }

  $3E07,    { Overflow 0011-1110        }

  $4109,    { Cell height  0100-0001    }
            { no line duplication       }
 {----------------------------------------}
  $EA10,    { V sync. start             }
            { + Overflow (7,2) = 256    }
            { 234+256 = Last Line 490   }
  $AC11,    { V sync. end/Prot CR0 CR7  }

  $DF12,    { Vertical displayed        }
            { + Overflow (6,1) = 768    }
            { 223+768 = 991             }
(*
  $4013,    { 2D offset                    } { 5A->180 40->128 }
            { 2*45 = 90 octets/line     }
*)
  $13 + (LineChars div 2) shl 8,

  $0014,    { DWord mode off            }
            { (offset *2)               }
  $E715,    { V Blank start             }
            { + overlow (3)             }
            { + CellHeight (6)          }
            { 231+512 = 743             }
  $0616,    { V Blank end               }
            { line 6 }
  $E317     { Turn on Byte mode         }

 );

 LineSize:word=LineChars;

Procedure TweakMode; assembler;
  asm
  { Bios mode 320x200 }
   mov ax,13h; int 10h

  { Tweak Mode 360x240 }

  { Disable Chain 4 Mode }
    mov dx,$3C4    { port[$3C4]:=4 }
    mov al,4
    out dx,al
    inc dx         { port[$3C5]:=(port[$3C5] and (not 8)) or 4 }
    in  al,dx
    and al,not 8
    or  al,4
    out dx,al
  (*
   mov   dx, $3C4               { Sequencer Register }
   mov   ax, $0604              { Disable Chain 4 Mode }
   out   dx, ax
  *)

  (*
   mov   ax, $0200              { (A)synchronous Reset }
   out   dx, ax
  *)


  {------------------------------------------------------}
   mov   dx, $3C2               { VGA Misc Register }
   mov   al, $E7                { Use 28Mhz Clock & 60Hz }
   out   dx, al
           { 1110-0111 - 720x400,Odd,RAM Enable,Color
             7 - vert retrace  \ vert resolution
             6 - horz retrace  / (reserve,350,400 lines)
             5 - odd
             4 - reserved
             3 - horz resolution (640,720,800)
             2 /
             1 - Enable RAM
             0 - monochrome
           }
  {------------------------------------------------------}

  (*
   mov   dx, $3C4               { Sequencer Register }
   mov   ax, $0200              { Restart Sequencer }
   out   dx, ax
  *)
   {
     Diasable Write protect For CrtC Registers 0-7, since we are
     about to change the horizontal & vertical timing settings.
   }
   mov   dx, $3D4               { VGA CrtC Registers }
   mov   al, $11                { CrtC register 11h }
   out   dx, al                 { Load current value }
   inc   dx                     { Point to data }
   in    al, dx                 { Get CrtC register 11h }
   and   al, $7F                { Mask out Write protect }
   out   dx, al                 { and send it back }

   { Send CrtC data in VMODE_DATA Array to the CrtC. }
   mov   dx, $3D4               { VGA CrtC Registers }
   cld                          { Forward block load }
   mov   si, offset TweakParm   { Get parameter data }
   mov   cx, 17                 { Number of entries in block }

   @@1:
     mov   ax, ds:[si]      { Get next parameter value }
     inc   si               { Advance to next Word }
     inc   si
     out   dx, ax           { Output next value }
     loop  @@1              { Process next value }

  { Clear Screen }
   mov dx,$3C4
   mov ax,$0F02
   out dx,ax
   mov ax,$0000
   mov es,SegA000
   xor di,di
   mov cx,360*240/4*3
   rep stosw

  { setup colors }
   mov dx,$3C8
   mov al,0
   out dx,al
   inc dx
   mov si,offset Palette
   mov cx,3*16
   rep outsb
 end;

Procedure TextMode; assembler;
 asm
  mov ax,03h; int 10h
 end;

Procedure SelectPage(Page:word); assembler;
 Const
  PageSize:word=360*240 div 4;
 asm
  mov ax,Page   (* select start of video address *)
  mul PageSize
  mov bx,ax

  shr ax,4
  add ax,SegA000
  mov VideoSeg,ax

  mov dx,$3D4
  mov al,$C
  mov ah,bh
  out dx,ax
  inc ax
  mov ah,bl
  out dx,ax
 end;

Procedure PutPixel(x,y,c:word); assembler;
 asm
  mov bx,x
  mov cx,bx
  shr bx,2
  and cx,3
  mov dx,$3c4
  mov ax,$0102;
  shl ah,cl
  out dx,ax
  mov ax,y
  mul LineSize
  add bx,ax
  mov es,VideoSeg
  mov ax,c
  mov es:[bx],al
 end;

Procedure HLine(x,y,w,c:word); assembler;
 asm
  mov ax,y
  mul LineSize
  mov bx,x
  mov cx,bx
  shr bx,2
  add ax,bx
  mov di,ax
  mov es,VideoSeg

  mov bx,w
  mov dx,$3c4

  { align left }
  and cx,3
  jz @RoundLeft
   mov ax,0F02h
   shl ah,cl
   out dx,ax
   mov ax,c
   mov es:[di],al
   inc di
   sub bx,4
   add bx,cx
  @RoundLeft:

  mov ax,$0F02
  out dx,ax

  mov ax,c
  mov ah,al
  mov cx,bx
  shr cx,2
  jz  @Skip
   rep stosb
  @Skip:

  { align Right }
  and bx,3
  jz @RoundRight
   mov cx,4
   sub cx,bx
   mov ax,0F02h
   shr ah,cl
   out dx,ax
   mov ax,c
   mov es:[di],al
  @RoundRight:
 end;

Procedure VLine(x,y,h,c:word); assembler;
 asm
  mov ax,y
  mul LineSize
  mov bx,x
  shr bx,2
  add ax,bx
  mov di,ax
  mov es,VideoSeg

  mov cx,x
  and cx,3
  mov ax,$0102
  shl ah,cl
  mov dx,$3c4
  out dx,ax

  mov ax,c
  mov cx,h
  @NextLine:
   mov es:[di],al
   add di,360/4
  loop @NextLine
 end;

Procedure Box(Left,Top,Width,Height,Color:word);
 var
  h:word;
 begin
  if Width=1 then
   VLine(Left,Top,Height,Color)
  else
   for h:=1 to Height do begin
    HLine(Left,Top,Width,Color);
    inc(top);
   end;
 end;

Procedure Frame3D(left,top,width,height,Light,Shadow,Background:word);
 begin
  Box(Left,Top,Width,Height,BackGround);
  HLine(Left,Top,Width,Light);
  VLine(Left,Top,Height,Light);
  HLine(Left,Top+Height-1,Width,Shadow);
  VLine(Left+Width-1,Top,Height,Shadow);
 end;

Type
 TRom8x8=array[#0..#255,0..7] of byte;

Var
 rom8x8:^Trom8x8;

Procedure OutTextXY(x,y,c:word;const Text:String);
 var
  is,ix,iy:integer;
  b:byte;
 begin
  is:=0;
 { While Text[is]<>#0 do begin }
  for is:=1 to length(Text) do begin

   for iy:=0 to 7 do begin
    b:=rom8x8^[text[is],iy];

    for ix:=0 to 7 do begin
     if b>127 then putpixel(x+ix,y+iy,c);
     b:=byte(b shl 1);
    end;

   end;

   inc(x,8);

 {  inc(is); }
  end;
 end;

begin
 asm
 push bp
  mov ax,1130h
  mov bh,03h
  int 10h
  mov word(rom8x8+0),bp
  mov word(rom8x8+2),es
 pop  bp
 end;
end.